import os
import unittest
from drb.drivers.zip import DrbZipFactory
from pathlib import Path
from drb.drivers.netcdf import DrbNetcdfFactory, DrbNetcdfGroupNode
from drb.drivers.file import DrbFileFactory


GROUP_ROOT = "root"
GROUP_ONE = "mozaic_flight_2012030319051051_descent"
GROUP_TWO = "mozaic_flight_2012030403540535_ascent"


class TestDrbGroupNodeNetcdf(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    zip_with_one_group = current_path / "files" / 'sgpsondewnpnC1.zip'
    name_netcdf_with_one_group = 'sgpsondewnpnC1.nc'

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_zip is not None:
            self.node_zip.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)

        self.node_zip = DrbZipFactory().create(self.node_file)
        self.node = DrbNetcdfFactory().create(self.node_zip[0])
        return self.node

    def test_first_group(self):

        node = self.open_node(str(self.zip_with_one_group))

        root_node = node[0]

        self.assertIsInstance(root_node, DrbNetcdfGroupNode)
        self.assertEqual(root_node.name, GROUP_ROOT)
        self.assertEqual(root_node.namespace_uri, None)

    def test_only_one_group(self):
        node = self.open_node(str(self.zip_with_one_group))

        root_node = node[0]

        self.assertTrue(root_node.has_child())
        self.assertEqual(len(root_node), 2)

        self.assertIsNotNone(root_node[0])
        self.assertEqual(root_node[0].name, "dimensions")

        self.assertIsNotNone(root_node[-1])
        self.assertEqual(root_node[-1].name, "variables")

        with self.assertRaises(IndexError):
            root_node[3]



import os
import unittest
from pathlib import Path


from drb.drivers.netcdf import DrbNetcdfFactory, DrbNetcdfGroupNode, \
    DrbNetcdfVariableNode
from drb.drivers.file import DrbFileFactory


GROUP_ROOT = "root"


class TestDrbNodeNetcdfHdf5(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))
    path_hdf5 = current_path / "files" / 'test.hdf5'

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbNetcdfFactory().create(self.node_file)
        return self.node

    def test_first_group(self):
        node = self.open_node(str(self.path_hdf5))

        root_node = node[0]

        self.assertIsInstance(root_node, DrbNetcdfGroupNode)
        self.assertEqual(root_node.name, GROUP_ROOT)
        self.assertEqual(root_node.namespace_uri, None)

    def test_first_var(self):
        node = self.open_node(str(self.path_hdf5))

        var_node = node[0]['variables'][0]

        self.assertIsInstance(var_node, DrbNetcdfVariableNode)

        self.assertEqual(var_node.name, 'default')



import enum
from abc import ABC
from typing import List, Any
import netCDF4 as netCDF
import numpy
import xarray

from deprecated.classic import deprecated
from drb.core import DrbNode
from drb.nodes.abstract_node import AbstractNode
from drb.topics import resolver


class DrbNetcdfAttributeNames(enum.Enum):
    UNLIMITED = 'unlimited'
    """
    A boolean indicating whether the netcdf file has a fixed size or not.
    """


class DrbNetcdfSimpleNode(AbstractNode, ABC):
    """
    This node will be inherited by DrbNetcdfSimpleValueNode
    and DrbNetcdfDimensionNode.

    It is used to represent the key to a node with value.

    Parameters:
        parent (DrbNode): The parent of the node.
        name (str): the name of the node.
    """

    def __init__(self, parent: DrbNode, name):
        super().__init__()
        self.parent: DrbNode = parent
        self.name = name

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    def children(self) -> List[DrbNode]:
        """
        This node as no children.

        Returns:
            List: An empty List
        """
        return []

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError


class DrbNetcdfSimpleValueNode(DrbNetcdfSimpleNode):
    """
    This node is used to get a simple value.

    Parameters:
        parent (DrbNode): The parent of the node.
        name (str): the name of the node.
        value (any): the value.
    """
    def __init__(self, parent: DrbNode, name: str, value: any):
        super().__init__(parent, name)
        self.value = value


class DrbNetcdfDimensionNode(DrbNetcdfSimpleNode):
    """
    This node is used to retrieve the dimension of a netcdf.
    A netCDF `Dimension` is used to describe the coordinates of a `Variable`.
    The value of the file is his dimension only if it is not UNLIMITED.

    Parameters:
        parent (DrbNode): The parent of the node.
        dimension (netCDF.Dimension): the dimension of the netcdf.
    """

    @staticmethod
    def __to_netcdf_dimension(node: DrbNode, **kwargs):
        if isinstance(node, DrbNetcdfDimensionNode):
            return node._dimension
        raise TypeError(f'invalid given node {node.__class__}')

    def __init__(self, parent: DrbNode, dimension: netCDF.Dimension):
        super().__init__(parent, dimension.name)
        self._dimension = dimension
        if self.value is None:
            if self._dimension.isunlimited():
                self.value = -1
            else:
                self.value = self._dimension.size
        self.__init_attributes()
        self.add_impl(netCDF.Dimension, self.__to_netcdf_dimension)

    def __init_attributes(self):
        """
        The attributes of this node only contain UNLIMITED a boolean,
        True if the netcdf has no limit otherwise False.
        """
        self @= (DrbNetcdfAttributeNames.UNLIMITED.value,
                 self._dimension.isunlimited())


class DrbNetcdfGroupNode(AbstractNode):
    """
    The DrbNetcdfGroupNode is used to organize data inside a netcdf file.

    The DrbNetcdfGroupNode can contain:

            **dimensions**: The `dimensions` dictionary maps the names of
            dimensions defined for the `Group` or `Dataset` to instances of the
            `Dimension` class.

            **variables**: The `variables` dictionary maps the names of
            variables defined for this `Dataset` or `Group` to instances
            of the `Variable` class.

            **groups**: The groups dictionary maps the names of groups created
            for this `Dataset` or `Group` to instances of the `Group` class
            (the `Dataset` class is simply a special case of the `Group` class
            which describes the root group in the netCDF4 file).

    Parameters:
        parent(DrbNode): The parent of the node.
        data_set(netCDF.Dataset):The dataset of the netcdf.
    """

    @staticmethod
    def __to_netcdf_dataset(node: DrbNode, **kwargs):
        if isinstance(node, DrbNetcdfGroupNode):
            return node._data_set
        raise TypeError(f'invalid given node {node.__class__}')

    @staticmethod
    def __to_xarray_dataset(node: DrbNode, **kwargs):
        if isinstance(node, DrbNetcdfGroupNode):
            store = xarray.backends.NetCDF4DataStore(node._data_set)
            return xarray.open_dataset(store)
        raise TypeError(f'invalid given node {node.__class__}')

    def __init__(self, parent: DrbNode, data_set: netCDF.Dataset):
        super().__init__()

        name = data_set.name
        if name == '/':
            name = 'root'
        self.name = name
        self.parent: DrbNode = parent
        self._children: List[DrbNode] = None
        self._data_set = data_set
        self.add_impl(netCDF.Dataset, self.__to_netcdf_dataset)
        self.add_impl(xarray.Dataset, self.__to_xarray_dataset)
        self.__init_attributes()

    def __init_attributes(self):
        for attribute_name in self._data_set.ncattrs():
            data = (attribute_name, getattr(self._data_set, attribute_name))
            self.__imatmul__(data)

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    @resolver.resolve_children
    def children(self) -> List[DrbNode]:
        if self._children is None:
            self._children = []
            dimensions = self._data_set.dimensions
            if dimensions is not None and len(dimensions) > 0:
                nodelist = DrbNetcdfListNode(self, 'dimensions')
                for dim in dimensions:
                    nodelist.append_child(
                        DrbNetcdfDimensionNode(nodelist, dimensions[dim]))
                self._children.append(nodelist)

            variables = self._data_set.variables
            if variables is not None and len(variables) > 0:
                nodelist = DrbNetcdfListNode(self, 'variables')
                for variable in variables:
                    nodelist.append_child(
                        DrbNetcdfVariableNode(nodelist, variables[variable]))
                self._children.append(nodelist)

            groups = self._data_set.groups
            for grp in groups.values():
                self._children.append(DrbNetcdfGroupNode(self, grp))
        return self._children

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError


class DrbNetcdfListNode(AbstractNode):
    """
    This node is used to have one or many children of DrbNode but no value.

    Parameters:
        parent (DrbNode): The node parent.
        name (str): the name of the data.
    """
    def __init__(self, parent: DrbNode, name: str):
        super().__init__()
        self.name = name
        self.parent: DrbNode = parent
        self._children: List[DrbNode] = []

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    @resolver.resolve_children
    def children(self) -> List[DrbNode]:
        return self._children

    def append_child(self, node: DrbNode) -> None:
        """
        Appends a DrbNode giving in argument to the list of children.

        Parameters:
            node (DrbNode): The node to add.
        """
        self._children.append(node)

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError


class DrbNetcdfVariableNode(AbstractNode):
    """
    This node is used to retrieve the variable of a netcdf.
    A netCDF `Variable` is used to read and write netCDF data.

    Parameters:
        parent (DrbNode): The parent of the node.
        variable (netCDF.Variable): the variable of the netcdf.
    """

    @staticmethod
    def __to_netcdf_variable(node: DrbNode, **kwargs):
        if isinstance(node, DrbNetcdfVariableNode):
            return node._variable
        raise TypeError(f'invalid given node {node.__class__}')

    @staticmethod
    def __to_xarray_data_array(node: DrbNode, **kwargs):
        if isinstance(node, DrbNetcdfVariableNode):
            grand_parent = node.parent.parent
            if grand_parent and grand_parent.has_impl(xarray.Dataset):
                xarray_dataset = grand_parent.get_impl(xarray.Dataset)
                return xarray_dataset[node.name]
            raise RuntimeError('Unsupported implementation: xarray.DataArray')
        raise TypeError(f'invalid given node {node.__class__}')

    @staticmethod
    def __to_numpy_nd_array(node: DrbNode, **kwargs):
        if isinstance(node, DrbNetcdfVariableNode):
            if node._is_scalar:
                node._variable.set_auto_mask(False)
                array_to_return = node._variable[:]
                node._variable.set_auto_mask(True)
                return array_to_return
            return node._variable[:]
        raise TypeError(f'invalid given node {node.__class__}')

    def __init__(self, parent: DrbNode, variable: netCDF.Variable):
        super().__init__()

        self.name = variable.name
        self.parent: DrbNode = parent
        self._children: List[DrbNode] = None
        self._variable = variable
        # value scalar indicate a variable with only one value
        # in this case value return this value
        # and all method to retrieve array are not activated
        # this type of variable can be for example a time...
        self._is_scalar = len(self._variable.shape) == 0

        self.add_impl(netCDF.Variable, self.__to_netcdf_variable)
        self.add_impl(xarray.DataArray, self.__to_xarray_data_array)
        if not self._is_scalar:
            self.add_impl(numpy.ndarray, self.__to_numpy_nd_array)
            if variable.mask:
                self.add_impl(numpy.ma.masked_array, self.__to_numpy_nd_array)

        self.__init_attributes()
        if self._is_scalar:
            self.value = self._variable.getValue()

    def __init_attributes(self):
        for attribute_name in self._variable.ncattrs():
            self @= (attribute_name, getattr(
                self._variable, attribute_name))

    @property
    @deprecated(version='1.2.0',
                reason='Usage of the bracket is recommended')
    def children(self) -> List[DrbNode]:

        if self._children is None:
            self._children = []
            if not self._is_scalar:
                self._children.append(DrbNetcdfSimpleValueNode(
                    self, 'dimensions', self._variable.dimensions))
                self._children.append(DrbNetcdfSimpleValueNode(
                    self, 'shape', self._variable.shape))
            self._children.append(DrbNetcdfSimpleValueNode(
                self, 'size', self._variable.size))
        return self._children

    def __setitem__(self, key, value):
        raise NotImplementedError

    def __delitem__(self, key):
        raise NotImplementedError



import os
import unittest
import uuid
from pathlib import Path

from drb.core.factory import FactoryLoader
from drb.nodes.logical_node import DrbLogicalNode
from drb.topics.dao import ManagerDao
from drb.topics.topic import TopicCategory

from drb.drivers.netcdf import DrbNetcdfFactory


class TestDrbNetcdfSignature(unittest.TestCase):
    mock_pkg = None
    fc_loader = None
    topic_loader = None
    netcdf_id = uuid.UUID('83720abe-2c0e-11ec-8d3d-0242ac130003')
    hdf5_id = uuid.UUID('b4e6ba07-3a59-4736-b33d-3c55dcae2c56')

    current_path = Path(os.path.dirname(os.path.realpath(__file__)))
    netcdf_with_one_group = current_path / "files" / 'sgpsondewnpnC1.nc'
    path_hdf5 = current_path / "files" / 'test.hdf5'

    empty_file = current_path / "files" / "empty.file"

    @classmethod
    def setUpClass(cls) -> None:
        cls.fc_loader = FactoryLoader()
        cls.topic_loader = ManagerDao()

    def test_impl_loading(self):
        factory_name = 'netcdf'

        factory = self.fc_loader.get_factory(factory_name)
        self.assertIsNotNone(factory)
        self.assertIsInstance(factory, DrbNetcdfFactory)

        topic = self.topic_loader.get_drb_topic(self.netcdf_id)
        self.assertIsNotNone(factory)
        self.assertEqual(self.netcdf_id, topic.id)
        self.assertEqual('netCDF', topic.label)
        self.assertIsNone(topic.description)
        self.assertEqual(TopicCategory.FORMATTING, topic.category)
        self.assertEqual(factory_name, topic.factory)

    def test_impl_signatures(self):
        topic = self.topic_loader.get_drb_topic(self.netcdf_id)

        node = DrbLogicalNode(self.netcdf_with_one_group)
        self.assertTrue(topic.matches(node))

        node = DrbLogicalNode(self.empty_file)
        self.assertFalse(topic.matches(node))

    def test_hdf5(self):
        topic = self.topic_loader.get_drb_topic(self.hdf5_id)

        node = DrbLogicalNode(self.path_hdf5)
        self.assertTrue(topic.matches(node))



import io
import os
import unittest
from pathlib import Path

import netCDF4

from drb.drivers.netcdf import DrbNetcdfFactory, DrbNetcdfAttributeNames
from drb.exceptions.core import DrbNotImplementationException
from drb.drivers.file import DrbFileFactory

GROUP_ROOT = "root"
GROUP_ONE = "mozaic_flight_2012030319051051_descent"
GROUP_TWO = "mozaic_flight_2012030403540535_ascent"

GROUP_NOT_EXIST = "fake_group"


class TestDrbDimensionNodeNetcdf(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    netcdf_fake = current_path / "files" / "fake.nc"
    netcdf_with_one_group = current_path / "files" / 'sgpsondewnpnC1.nc'
    netcdf_with_multiple_groups = current_path / "files" / 'test_hgroups.nc'

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbNetcdfFactory().create(self.node_file)
        return self.node

    def test_dimension_unlimited(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]
        self.assertIsNotNone(dim_list["time"])

        dim_time = dim_list["time"]
        self.assertEqual(dim_time.value, -1)

        list_attribute = dim_time.attributes
        self.assertEqual(
            list_attribute[(DrbNetcdfAttributeNames.UNLIMITED.value, None)],
            True)

        self.assertTrue(dim_time.get_attribute(
            DrbNetcdfAttributeNames.UNLIMITED.value, None))

    def test_dimension_limited(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        first_group = root_node['mozaic_flight_2012030319051051_descent']

        dim_list = first_group["dimensions"]
        self.assertIsNotNone(dim_list["air_press"])

        dim_time = dim_list["air_press"]
        self.assertEqual(dim_time.value, 78)

        list_attribute = dim_time.attributes
        self.assertEqual(
            list_attribute[(DrbNetcdfAttributeNames.UNLIMITED.value, None)],
            False)

        self.assertFalse(dim_time.get_attribute(
            DrbNetcdfAttributeNames.UNLIMITED.value, None))

    def test_dimension_no_children(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        dim_time = dim_list["time"]

        with self.assertRaises(IndexError):
            dim_time[0]

        with self.assertRaises(KeyError):
            dim_time["toto"]

        self.assertEqual(len(dim_time.children), 0)
        self.assertFalse(dim_time.has_child())
        self.assertEqual(len(dim_time), 0)

    def test_dimension_parent(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        dim_time = dim_list[0]

        self.assertEqual(dim_time.parent, dim_list)

        self.assertEqual(dim_time.name, "time")

    def test_dimension_impl(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        dim_time = dim_list["time"]

        self.assertTrue(dim_time.has_impl(netCDF4.Dimension))

        dim_nc = dim_time.get_impl(netCDF4.Dimension)

        self.assertIsInstance(dim_nc, netCDF4.Dimension)

    def test_dimension_impl_not_supported(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        dim_time = dim_list["time"]

        self.assertFalse(dim_time.has_impl(io.BufferedIOBase))

        with self.assertRaises(DrbNotImplementationException):
            dim_time.get_impl(io.BufferedIOBase)

    def test_dimension_namespace_uri(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        dim_time = dim_list["time"]

        self.assertEqual(dim_time.namespace_uri, None)



import io
import os
import unittest
from pathlib import Path

import netCDF4
import xarray


from drb.drivers.netcdf import DrbNetcdfFactory, DrbNetcdfGroupNode
from drb.exceptions.core import DrbNotImplementationException
from drb.drivers.file import DrbFileFactory

GROUP_ROOT = "root"
GROUP_ONE = "mozaic_flight_2012030319051051_descent"
GROUP_TWO = "mozaic_flight_2012030403540535_ascent"

GROUP_NOT_EXIST = "fake_group"


class TestDrbGroupNodeNetcdf(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    netcdf_fake = current_path / "files" / "fake.nc"
    netcdf_with_one_group = current_path / "files" / 'sgpsondewnpnC1.nc'
    netcdf_with_multiple_groups = current_path / "files" / 'test_hgroups.nc'
    netcdf_with_sample_groups = current_path / "files" / 'sample.nc'

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbNetcdfFactory().create(self.node_file)
        return self.node

    def test_first_group(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        self.assertIsInstance(root_node, DrbNetcdfGroupNode)
        self.assertEqual(root_node.name, GROUP_ROOT)
        self.assertEqual(root_node.namespace_uri, None)

    def test_only_one_group(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        self.assertTrue(root_node.has_child())
        self.assertEqual(len(root_node), 2)

        self.assertIsNotNone(root_node[0])
        self.assertEqual(root_node[0].name, "dimensions")

        self.assertIsNotNone(root_node[-1])
        self.assertEqual(root_node[-1].name, "variables")

        with self.assertRaises(IndexError):
            root_node[3]

    def test_only_group_with_sub_group(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        self.assertTrue(root_node.has_child())
        self.assertTrue(len(root_node) > 2)

        self.assertIsNotNone(root_node[GROUP_ONE])
        self.assertIsNotNone(root_node[GROUP_ONE])
        group_one = root_node[GROUP_ONE]
        self.assertEqual(group_one.name, GROUP_ONE)

        with self.assertRaises(KeyError):
            root_node[(GROUP_ONE, 2)]

        self.assertIsNotNone(root_node[(GROUP_TWO, 0)])
        with self.assertRaises(KeyError):
            root_node[(GROUP_TWO, 2)]

    def test_multiples_sub_group(self):
        node = self.open_node(str(self.netcdf_with_sample_groups))

        root_node = node[0]
        level_node_1 = root_node["Test_Drb_Group_A"]
        self.assertIsNotNone(level_node_1)
        level_node_2 = level_node_1["Test_Drb_Sub_Group_A1"]
        self.assertIsNotNone(level_node_2)

    def test_group_attributes(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]
        list_attributes = root_node.attributes

        self.assertEqual(len(list_attributes), 27)

        self.assertEqual(list_attributes[('phase_fitting_1', None)],
                         'Phase fitting length is  60 s from   0 min to'
                         ' 120 min\r\n')

        with self.assertRaises(KeyError):
            list_attributes[('phase_fitting_1', 'test')]

        self.assertEqual(root_node.get_attribute('zeb_platform'),
                         'sgpsondewnpnC1.a1')

    def test_group_attributes_sub_grp(self):
        node = self.open_node(str(self.netcdf_with_sample_groups))

        root_node = node[0]
        list_attributes = root_node.attributes

        self.assertEqual(len(list_attributes), 0)

        level_node_1 = root_node[("Test_Drb_Group_A", 0)]
        list_attributes = level_node_1.attributes

        self.assertEqual(len(list_attributes), 2)
        self.assertEqual(level_node_1.get_attribute('first_attribute'),
                         'attributes test A')

        level_node_2 = level_node_1["Test_Drb_Sub_Group_A1"]

        list_attributes = level_node_2.attributes

        self.assertEqual(len(list_attributes), 2)
        self.assertEqual(level_node_2.get_attribute('first_attribute'),
                         'attributes test A1')

        level_node_2b = level_node_1["Test_Drb_Sub_Group_A2"]

        list_attributes = level_node_2b.attributes

        self.assertEqual(len(list_attributes), 2)
        self.assertEqual(level_node_2b.get_attribute('first_attribute'),
                         'attributes test B')

        level_node_1b = root_node[-1]
        list_attributes = level_node_1b.attributes
        self.assertEqual(len(list_attributes), 0)

        level_node_2b = level_node_1b["Test_Drb_Sub_Group_B1"]

        self.assertEqual(level_node_2b.get_attribute('first_attribute'),
                         'attributes test B1')

    def test_group_parent(self):
        node = self.open_node(str(self.netcdf_with_sample_groups))

        root_node = node[0]

        self.assertEqual(root_node.parent, node)
        level1 = root_node[0]

        self.assertEqual(level1.parent, root_node)

    def test_group_value(self):
        node = self.open_node(str(self.netcdf_with_sample_groups))

        root_node = node[0]

        self.assertIsNone(root_node.value)

    def test_group_get_children_at(self):
        node = self.open_node(str(self.netcdf_with_sample_groups))

        root_node = node[0]
        list_attributes = root_node.attributes

        self.assertEqual(len(list_attributes), 0)

        level_node_1 = root_node[0]
        self.assertIsNotNone(level_node_1)
        self.assertEqual(level_node_1.name, "Test_Drb_Group_A")

        with self.assertRaises(IndexError):
            root_node[5]
        self.assertEqual(root_node[-2], root_node[len(root_node) - 2])

    def test_group_impl(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]
        first_child = root_node[GROUP_ONE]

        self.assertTrue(first_child.has_impl(netCDF4.Dataset))

        netcdf_dataset = first_child.get_impl(netCDF4.Dataset)

        self.assertIsInstance(netcdf_dataset, netCDF4.Dataset)

    def test_group_impl_xarray(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]
        first_child = root_node[GROUP_ONE]

        self.assertTrue(first_child.has_impl(xarray.Dataset))

        netcdf_dataset = first_child.get_impl(xarray.Dataset)

        self.assertIsInstance(netcdf_dataset, xarray.Dataset)

    def test_dimension_impl_not_supported(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))
        root_node = node[0]

        self.assertIsInstance(root_node, DrbNetcdfGroupNode)
        self.assertFalse(root_node.has_impl(io.BufferedIOBase))

        with self.assertRaises(DrbNotImplementationException):
            root_node.get_impl(io.BufferedIOBase)



from . import _version
from .netcdf import (
    DrbNetcdfGroupNode, DrbNetcdfVariableNode,
    DrbNetcdfAttributeNames, DrbNetcdfListNode,
    DrbNetcdfDimensionNode, DrbNetcdfSimpleValueNode)

from .netcdf_node_factory import DrbNetcdfFactory, DrbNetcdfNode

__version__ = _version.get_versions()['version']


del _version

__all__ = [
    'DrbNetcdfNode',
    'DrbNetcdfFactory',
    'DrbNetcdfGroupNode',
    'DrbNetcdfVariableNode',
    'DrbNetcdfAttributeNames',
    'DrbNetcdfListNode',
    'DrbNetcdfDimensionNode',
    'DrbNetcdfSimpleValueNode',
]



import os
import io
import unittest
from pathlib import Path


import netCDF4
from drb.drivers.netcdf import DrbNetcdfFactory, DrbNetcdfAttributeNames
from drb.drivers.file import DrbFileFactory
from drb.exceptions.core import DrbNotImplementationException, DrbException


GROUP_ROOT = "root"
GROUP_ONE = "mozaic_flight_2012030319051051_descent"
GROUP_TWO = "mozaic_flight_2012030403540535_ascent"

GROUP_NOT_EXIST = "fake_group"


class TestDrbListNodeNetcdf(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    netcdf_with_one_group = current_path / "files" / 'sgpsondewnpnC1.nc'

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbNetcdfFactory().create(self.node_file)
        return self.node

    def test_list_children(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        self.assertTrue(dim_list.has_child())

    def test_list_get_named_children(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]
        dim_list = root_node["dimensions"]

        self.assertIsNotNone(dim_list["time"])
        with self.assertRaises(KeyError):
            dim_list[("time", 2)]

        list_children = dim_list["time", None, :]
        self.assertEqual(list_children[0].name, 'time')

    def test_list_children_get_at(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        self.assertEqual(len(dim_list), 1)

        self.assertIsNotNone(dim_list[0])
        self.assertIsNotNone(dim_list[-1])

        with self.assertRaises(IndexError):
            dim_list[2]

        with self.assertRaises(KeyError):
            dim_list["toto"]

        self.assertEqual(dim_list[0], dim_list[-1])

        self.assertEqual(len(dim_list.children), 1)
        self.assertTrue(dim_list.has_child())

    def test_list_parent(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]
        dim_list = root_node["dimensions"]
        self.assertEqual(dim_list.parent, root_node)
        self.assertEqual(dim_list.name, "dimensions")

    def test_list_namespace_uri(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        self.assertEqual(dim_list.namespace_uri, None)

    def test_list_value(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        self.assertIsNone(dim_list.value)

    def test_list_impl_not_supported(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]

        self.assertFalse(dim_list.has_impl(io.BufferedIOBase))
        self.assertFalse(dim_list.has_impl(netCDF4.Dimension))

        with self.assertRaises(DrbNotImplementationException):
            dim_list.get_impl(io.BufferedIOBase)

    def test_list_no_attributes(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        dim_list = root_node["dimensions"]
        self.assertFalse(bool(dim_list.attributes))
        with self.assertRaises(DrbException):
            dim_list.get_attribute(DrbNetcdfAttributeNames.UNLIMITED.value,
                                   None)



import io
import os
import unittest
from pathlib import Path

import netCDF4
import numpy
import xarray
from drb.drivers.netcdf import DrbNetcdfFactory, DrbNetcdfSimpleValueNode
from drb.exceptions.core import DrbNotImplementationException, DrbException
from drb.drivers.file import DrbFileFactory


GROUP_ROOT = "root"
GROUP_ONE = "mozaic_flight_2012030319051051_descent"
GROUP_TWO = "mozaic_flight_2012030403540535_ascent"

GROUP_NOT_EXIST = "fake_group"


class TestDrbVaraibleNodeNetcdf(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    netcdf_fake = current_path / "files" / "fake.nc"
    netcdf_with_one_group = current_path / "files" / 'sgpsondewnpnC1.nc'
    netcdf_with_multiple_groups = current_path / "files" / 'test_hgroups.nc'
    netcdf_with_sample_groups = current_path / "files" / 'sample.nc'

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbNetcdfFactory().create(self.node_file)
        return self.node

    def test_variable_dimension_shape(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        group_two = root_node[GROUP_TWO]
        list_var = group_two['variables']

        var_air_press = list_var['air_press']
        self.assertIsNotNone(var_air_press)

        list_dim = var_air_press['dimensions']
        self.assertIsNotNone(list_dim)
        self.assertIn('recNum', list_dim.value)

        list_shape = var_air_press['shape']
        self.assertIsInstance(list_shape, DrbNetcdfSimpleValueNode)
        self.assertIn(74, list_shape.value)

    def test_variable_attributes(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        list_var = root_node['variables']

        var_air_press = list_var[('rh', 0)]
        self.assertIsNotNone(var_air_press)

        attributes = var_air_press.attributes
        self.assertEqual(len(attributes), 4)

        self.assertIsNotNone(var_air_press.get_attribute('units', None))
        attr = var_air_press.get_attribute('units', None)
        self.assertEqual(attr, '%')
        attr = var_air_press.get_attribute('missing_value', None)
        self.assertEqual(attr, -9999.0)

        with self.assertRaises(DrbException):
            self.assertIsNotNone(var_air_press.get_attribute('toto', None))

    def test_variable_parent(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        group_two = root_node[GROUP_TWO]
        list_var = group_two['variables']

        var_air_press = list_var['air_press']

        self.assertEqual(var_air_press.parent, list_var)

    def test_variable_impl(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        group_two = root_node[GROUP_TWO]
        list_var = group_two['variables']

        var_air_press = list_var['air_press']

        self.assertTrue(var_air_press.has_impl(netCDF4.Variable))

        impl = var_air_press.get_impl(netCDF4.Variable)

        self.assertIsInstance(impl, netCDF4.Variable)

    def test_variable_xarray_impl(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        group_two = root_node[GROUP_TWO]
        list_var = group_two['variables']

        var_air_press = list_var['air_press']

        self.assertTrue(var_air_press.has_impl(xarray.DataArray))

        impl = var_air_press.get_impl(xarray.DataArray)

        self.assertIsInstance(impl, xarray.DataArray)

    def test_variable_impl_masked_array(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        group_two = root_node[GROUP_TWO]
        list_var = group_two['variables']

        var_air_press = list_var['air_press']

        self.assertTrue(var_air_press.has_impl(numpy.ma.masked_array))

        impl = var_air_press.get_impl(numpy.ma.masked_array)

        self.assertIsInstance(impl, numpy.ma.masked_array)
        self.assertIsInstance(impl, numpy.ma.core.MaskedArray)
        self.assertIsInstance(impl, numpy.ndarray)

        impl = var_air_press.get_impl(numpy.ndarray)

        self.assertEqual(impl[9], 85338)

        var_air_press.close()

    def test_variable_impl_not_supported(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        group_two = root_node[GROUP_TWO]
        list_var = group_two['variables']

        var_air_press = list_var['air_press']

        self.assertFalse(var_air_press.has_impl(io.BufferedIOBase))

        with self.assertRaises(DrbNotImplementationException):
            var_air_press.get_impl(io.BufferedIOBase)

    def test_variable_namespace_uri(self):
        node = self.open_node(str(self.netcdf_with_multiple_groups))

        root_node = node[0]

        group_two = root_node[GROUP_TWO]
        list_var = group_two['variables']

        var_air_press = list_var['air_press']

        self.assertEqual(var_air_press.namespace_uri, None)

    def test_variable_get_first_last_children(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        list_var = root_node['variables']

        var_wind = list_var[('u_wind', 0)]

        first_child_node = var_wind[0]
        self.assertIsNotNone(first_child_node)

        self.assertEqual(first_child_node.name, 'dimensions')

        lest_child_node = var_wind[-1]
        self.assertIsNotNone(lest_child_node)
        self.assertEqual(lest_child_node.name, 'size')

    def test_variable_scalar(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        list_var = root_node['variables']

        var_base_time = list_var[('base_time', 0)]

        self.assertIsNotNone(var_base_time)
        self.assertEqual(len(var_base_time), 1)

        with self.assertRaises(KeyError):
            var_base_time['dimensions']

        self.assertEqual(var_base_time.value, 1020770640)

        self.assertTrue(var_base_time.has_impl(netCDF4.Variable))

        self.assertFalse(var_base_time.has_impl(numpy.ma.masked_array))

    def test_variable_not_scalar(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        list_var = root_node['variables']

        var_lon = list_var['lon']

        self.assertIsNotNone(var_lon)
        self.assertEqual(len(var_lon), 3)

        dimensions_node = var_lon['dimensions']
        self.assertIsNotNone(dimensions_node)
        self.assertEqual(len(dimensions_node.value), 1)
        self.assertEqual(dimensions_node.value[0], 'time')

        shape_node = var_lon['shape']
        self.assertIsNotNone(shape_node)
        self.assertEqual(len(shape_node.value), 1)
        self.assertEqual(shape_node.value[0], 2951)

        self.assertEqual(var_lon.value, None)

    def test_variable_simple_get_children(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        list_var = root_node['variables']

        var_lon = list_var['lon']
        dimensions_node = var_lon['dimensions']
        self.assertIsNotNone(dimensions_node)
        with self.assertRaises(IndexError):
            dimensions_node[0]

        with self.assertRaises(IndexError):
            dimensions_node[-1]

            dimensions_node[0]

    def test_variable_simple_attributes(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        list_var = root_node['variables']

        var_lon = list_var['lon']
        self.assertTrue(var_lon.has_child())
        dimensions_node = var_lon['dimensions']

        self.assertEqual(len(dimensions_node.attributes), 0)
        with self.assertRaises(DrbException):
            dimensions_node.get_attribute('units', None)

    def test_variable_has_not_impl(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        list_var = root_node['variables']

        var_lon = list_var['lon']
        dimensions_node = var_lon['dimensions']

        self.assertFalse(dimensions_node.has_impl(io.BufferedIOBase))
        self.assertFalse(dimensions_node.has_impl(netCDF4.Variable))

        with self.assertRaises(DrbNotImplementationException):
            dimensions_node.get_impl(netCDF4.Variable)



import io
import os
import tempfile
from typing import Any, List, Dict, Tuple, Optional

import netCDF4

from drb.nodes.abstract_node import AbstractNode
from drb.drivers.file import DrbFileNode
from drb.core import DrbNode, ParsedPath, DrbFactory

from . import DrbNetcdfGroupNode
from ...exceptions.netcdf import DrbNetcdfNodeException


class DrbNetcdfNode(AbstractNode):
    """
    This node is used to instantiate a DrbNetcdfNode from another
    implementation of drb such as file.


    Parameters:
        base_node (DrbNode): the base node of this node.
    """
    def __init__(self, base_node: DrbNode):
        super().__init__()

        _netcdf_file_source = None
        self._root_dataset = None
        self.base_node = base_node
        self.temp_file = False
        stream_io = None
        if isinstance(self.base_node, DrbFileNode):
            _netcdf_file_source = self.base_node \
                         .get_impl(io.BufferedIOBase)
        else:
            if self.base_node.has_impl(io.BytesIO):
                stream_io = self.base_node.get_impl(io.BytesIO)
            elif self.base_node.has_impl(io.BufferedIOBase):
                stream_io = self.base_node.get_impl(io.BufferedIOBase)

            if stream_io is not None:
                _netcdf_file_source = tempfile.NamedTemporaryFile(delete=False)
                _netcdf_file_source.write(stream_io.read())
                stream_io.close()
                self.temp_file = True
            else:
                raise DrbNetcdfNodeException(f'Unsupported parent '
                                             f'{type(self.base_node).__name__}'
                                             f' for DrbNetcdfRootNode')
        self._filename = _netcdf_file_source.name
        _netcdf_file_source.close()
        self._root_node = None

    @property
    def parent(self) -> Optional[DrbNode]:
        return self.base_node.parent

    @property
    def path(self) -> ParsedPath:
        return self.base_node.path

    @property
    def name(self) -> str:
        return self.base_node.name

    @property
    def namespace_uri(self) -> Optional[str]:
        return self.base_node.namespace_uri

    @property
    def value(self) -> Optional[Any]:
        return self.base_node.value

    @property
    def attributes(self) -> Dict[Tuple[str, str], Any]:
        return self.base_node.attributes

    def get_attribute(self, name: str, namespace_uri: str = None) -> Any:
        return self.base_node.get_attribute(name, namespace_uri)

    @property
    def children(self) -> List[DrbNode]:
        if self._root_node is None:
            self._root_dataset = netCDF4.Dataset(filename=self._filename)
            self._root_node = DrbNetcdfGroupNode(self, self._root_dataset)
        return [self._root_node]

    def has_impl(self, impl: type) -> bool:
        return self.base_node.has_impl(impl)

    def get_impl(self, impl: type, **kwargs) -> Any:
        return self.base_node.get_impl(impl)

    def close(self):
        if self._root_dataset is not None:
            self._root_dataset.close()
        if self.temp_file:
            os.remove(self._filename)
        self.base_node.close()


class DrbNetcdfFactory(DrbFactory):

    def _create(self, node: DrbNode) -> DrbNode:
        if isinstance(node, DrbNetcdfNode):
            return node
        return DrbNetcdfNode(base_node=node)



import os
import posixpath
import unittest
from pathlib import Path
import io
from drb.drivers.netcdf import DrbNetcdfFactory, DrbNetcdfNode, \
    DrbNetcdfGroupNode
from drb.drivers.file import DrbFileFactory

ROOT_GROUP = "root"
GROUP_NOT_EXIST = "fake_group"


class TestDrbNodeFactoryNetcdf(unittest.TestCase):
    current_path = Path(os.path.dirname(os.path.realpath(__file__)))

    netcdf_fake = current_path / "files" / "fake.nc"
    netcdf_with_one_group = current_path / "files" / 'sgpsondewnpnC1.nc'
    netcdf_with_multiple_groups = current_path / "files" / 'test_hgroups.nc'

    def setUp(self) -> None:
        self.node = None
        self.node_file = None

    def tearDown(self) -> None:
        if self.node is not None:
            self.node.close()
        if self.node_file is not None:
            self.node_file.close()

    def open_node(self, path_file):
        self.node_file = DrbFileFactory().create(path_file)
        self.node = DrbNetcdfFactory().create(self.node_file)
        return self.node

    def test_opened_file_node(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        self.assertIsInstance(node, DrbNetcdfNode)
        self.assertEqual(node.name, self.node_file.name)
        self.assertEqual(node.namespace_uri, self.node_file.namespace_uri)

    def test_base_node(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        self.assertEqual(node.parent, self.node_file.parent)
        self.assertEqual(node.value, self.node_file.value)

        self.assertIsInstance(node, DrbNetcdfNode)

        self.assertEqual(len(node), 1)
        self.assertTrue(node.has_child())

    def test_base_node_get_child(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        self.assertEqual(node[0].name, ROOT_GROUP)
        self.assertEqual(len(node.children), 1)

    def test_base_node_attribute(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        self.assertEqual(node.attributes, self.node_file.attributes)

        self.assertEqual(node.get_attribute('mode'),
                         self.node_file.get_attribute('mode'))

    def test_base_node_impl(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        impl_base_file = io.BufferedIOBase

        self.assertTrue(node.has_impl(impl_base_file))

        impl = node.get_impl(impl_base_file)
        self.assertIsNotNone(impl)
        self.assertIsInstance(impl, impl_base_file)

    def test_first_group(self):
        node = self.open_node(str(self.netcdf_with_one_group))
        root_node = node[0]

        self.assertIsInstance(root_node, DrbNetcdfGroupNode)
        self.assertEqual(root_node.name, ROOT_GROUP)

    def test_path(self):
        node = self.open_node(str(self.netcdf_with_one_group))

        root_node = node[0]

        self.assertEqual(root_node.path.path, node.path.path + posixpath.sep
                         + 'root')
